<?php
/**
 * Form constructor for the Entities administration form.
 */

/**
 * Return the value to display on cell of the entity admin table.
 *
 * Inspired fy the node_admin_nodes() core function.
 *
 * @param $entity_type
 *   String, the entity type
 * @param $entity
 *   An entity object.
 * @param $header
 *   String, the colomn name, one from entity_admin_properties() ('title', etc...)
 * @param $datas
 *   Array, values from the entity database table, with at least the keys
 *   'entity_type' and 'entity_id', 'name' if available.
 *
 * @return
 *   String to display into the table cell.
 */
function entity_admin_entity_row($entity_type, $entity, $header, $datas) {
  $info = entity_admin_entity_get_info($entity_type);
  $empty_col = '';
  $property_field = (!empty($info['entity admin']['properties'][$header])) ? $info['entity admin']['properties'][$header] : FALSE;
  $col = FALSE;
  switch ($header) {
    case 'entity_type':
      $col = $info['label'];
      break;
    case 'bundle':
      $col = ($property_field) ? ((!empty($info['bundles'][$entity->$property_field]['label'])) ? $info['bundles'][$entity->$property_field]['label'] : t('@bundle (deleted)', array('@bundle' => $entity->$property_field)))  : $empty_col;
      break;
    case 'title':
      $entity_label = entity_admin_entity_label($entity_type, $entity); //(!empty($datas['entity_label'])) ? $datas['entity_label'] : t('no title');
      $entity_path = entity_admin_entity_path($entity_type, $entity);
      if ($entity_path) { //!empty(($datas['entity_path']))) {
        $languages = language_list();
        $langcode = entity_language($entity_type, $entity);
        $l_options = $langcode != LANGUAGE_NONE && isset($languages[$langcode]) ? array('language' => $languages[$langcode]) : array();
        $col = array(
          'data' => array(
            '#type' => 'link',
            '#title' => $entity_label,
            '#href' => $entity_path,
            '#options' => $l_options,
          ),
        );
      }
      else {
        $col = $entity_label;
      }
      break;
    case 'history':
      // Handle the mark
      if ($property_field) {
        //$ts_change = $entity->$property_field;
        $col = theme('mark', array('type' => entity_admin_mark($entity_type, $entity)));
      }
      else {
        $col = $empty_col;
      }
      break;
    case 'author':
      $col = ($entity_type != 'user') ? (($property_field) ? str_replace(' (not verified)', '', theme('username', array('account' => user_load($entity->$property_field)))) : $empty_col) : $empty_col;
      break;
    case 'status':
      if (in_array($entity_type, array('commerce_order', 'commerce_line_item'))) {
        $commerce_status = FALSE;
        if ($entity_type == 'commerce_line_item') {
          // Status of a commerce_line_item is the one from commerce_order.
          if ($commerce_order = commerce_order_load($entity->order_id)) {
            $commerce_status = $commerce_order->status;
          }
        }
        else {
          $commerce_status = $entity->$property_field;
        }
        if ($commerce_status === FALSE) {
          $col = t('orphan');
        }
        else {
          $order_statuses = commerce_order_statuses();
          if (isset($order_statuses[$commerce_status]['title'])) {
            $col = $order_statuses[$commerce_status]['title'];
          }
          else {
            $col = $empty_col;
          }
        }
      }
      elseif (in_array($entity_type, array('node', 'comment'))) {
        $col = ($entity->status) ? t('published') : t('not published');
      }
      elseif ((is_string($property_field) && $property_field == 'disabled') || $entity_type == 'views') {
        $col = (!$entity->disabled) ? t('enabled') : t('disabled');
      }
      elseif (!empty($info['base table field types'][$property_field]) && $info['base table field types'][$property_field] != 'int') {
        // Status col store strings, not numeric. Display the string.
        // This case exists for registration entities.
        $col = $entity->$property_field;
      }
      else {
        $col = ($property_field) ? (($entity->$property_field) ? t('actived') : t('not actived')) : $empty_col;
      }

      break;
    case 'changed':
      $changed = NULL;
      if (is_string($property_field)) {
        if (isset($entity->$property_field)) {
          $changed = $entity->$property_field;
        }
        elseif (isset($datas['changed'])) {
          $changed = $datas['changed'];
        }
        else {
          $changed = entity_admin_last_changed($entity_type, $entity);
        }
      }
      $col = (!empty($changed)) ? format_date($changed, 'short') : $empty_col;
      break;
    case 'created':
      $col = ($property_field) ? format_date($entity->$property_field, 'short') : $empty_col;
      break;
    case 'language':
      if (!$property_field) {
        $col = $empty_col;
      }
      else {
        $languages = language_list();
        $langcode = entity_language($entity_type, $entity);
        if ($entity_type == 'user' && empty($langcode)) {
          // For users, entity_language() fails.
          $langcode = $entity->language;
        }
        if ($langcode == LANGUAGE_NONE || isset($languages[$langcode])) {
          $col = ($langcode == LANGUAGE_NONE) ? t('Language neutral') : t($languages[$langcode]->name);
        }
        else {
          $col = (!empty($langcode)) ? t('Undefined language (@langcode)', array('@langcode' => $langcode)) : t('Undefined language');
        }
      }
      break;
    case 'entity_status':
      $col = ($property_field) ? theme('entity_status', array('status' => $entity->$property_field)) : $empty_col;
      break;
    default:


  }

  return $col;
}

/**
 * Return path for an operation.
 *
 * Inspired fy the node_admin_nodes() core function.
 *
 * @param $entity_type
 *   String, the entity type
 * @param $entity
 *   An entity object.
 * @param $name
 *   String, the link name, one from entity_admin_page_link_operations() ('edit',
 *   etc...)
 * @param $entity_path
 *   String, the entity view path.
 *
 * @return
 *   String, path of this operation.
 */
function entity_admin_page_links($entity_type, $entity, $name, $entity_path) {
  $info = entity_admin_entity_get_info($entity_type);
  $fixed_status = entity_admin_entity_has_status($entity_type, $entity, ENTITY_FIXED);
  $path = FALSE;
  switch ($name) {
    case 'clone':
      if (($fixed_status || $entity_type == 'views') && entity_admin_check_access('update', $entity_type, $entity)) {
        $path = $entity_path . '/clone';
      }
      break;
    case 'edit':
      if (!$fixed_status && entity_admin_check_access('update', $entity_type, $entity)) {
        // No 'edit' for rules_config
        $path_array = explode('/', $entity_path);
        $path_id = array_pop($path_array);
        $path_prefix_id = array_pop($path_array);
        $path = ($path_prefix_id != 'manage') ? $entity_path . '/edit' : $entity_path;
      }
      break;
    case 'reply':
      if ($entity_type == 'comment' && node_access('view', $entity->nid)) {
        // $entity_path is 'comment/1'. Return comment/reply/<nid>/1.
        $path = 'comment/reply/' . $entity->nid . '/' . $entity->cid;
      }
      break;
    case 'fields':
    case 'display':
      if (!$fixed_status) {
        $field_ui = !empty($info['bundle of']) && entity_type_is_fieldable($info['bundle of']) && module_exists('field_ui');
        if ($field_ui && user_access('administer fields')) {
          // retreive the sub-entity type
          $sub_entity_info = entity_admin_entity_get_info($info['bundle of']);
          $field_path = FALSE;
          $entity_name = (!empty($info['entity keys']['name']) && isset($entity->{$info['entity keys']['name']})) ? $entity->{$info['entity keys']['name']} : '';
          foreach ($sub_entity_info['bundles'] as $bundle_name => $bundle_datas) {
            if ($bundle_name == $entity_name) {
              if (isset($bundle_datas['admin'])) {
                // Extract path information from the bundle.
                if (!empty($bundle_datas['admin']['real path'])) {
                  $field_path = $bundle_datas['admin']['real path'];
                }
                elseif (!empty($bundle_datas['admin']['path'])) {
                  $path_array = explode('/', $bundle_datas['admin']['path']);
                  $path_array[$bundle_datas['admin']['bundle argument']] = $entity_name;
                  $field_path = implode('/', $path_array);
                }
              }
              break;
            }
          }
          if (!$field_path) {
            $field_path = $entity_path;
          }
          if ($field_path) {
            $path = $field_path . '/' . $name;
          }
        }
      }
      break;
    case 'translate':
      if (!$fixed_status) {
        if (in_array('language', $info['schema_fields_sql']['base table'])) {
          if (($entity_type == 'node' && module_exists('translation') && _translation_tab_access($entity))
              || ($entity_type == 'taxonomy_term' && module_exists('i18n_taxonomy') && i18n_taxonomy_term_translation_tab_access($entity))
              || (!empty($info['translation']['entity_translation']) && module_exists('entity_translation') && entity_translation_tab_access($entity_type, $entity))
          ) {
            $path = $entity_path . '/translate';
          }
        }
      }
      break;
    case 'delete':
      $views_in_code = ($entity_type == 'views' && entity_admin_entity_has_status($entity_type, $entity, ENTITY_IN_CODE));
      if (!$fixed_status && !$views_in_code && entity_admin_check_access('delete', $entity_type, $entity)) {
        if (in_array($entity_type, array('taxonomy_vocabulary', 'taxonomy_term'))) {
          // Taxonomy does not provide a deletion path. Deleted from edit page.
          $path = $entity_path . '/edit';

        }
        else {
          $path = $entity_path . '/delete';
        }
      }
      break;
    case 'revert':
      if (entity_admin_entity_has_status($entity_type, $entity, ENTITY_OVERRIDDEN) && entity_admin_check_access('update', $entity_type, $entity)) {
        $path = $entity_path . '/revert';
      }
      break;
    case 'export':
      if (!empty($info['exportable']) && entity_admin_check_access('view', $entity_type, $entity)) {
        $path = $entity_path . '/export';
      }
      break;
  }

  if ($path) {
    return $path;
  }
  return FALSE;
}

/**
 * Determines whether the given user can perform actions on an entity.
 *
 * Fork of the entity_access() function bu use the entity_admin_get_entity_info()
 * instead of entity_get_info.
 *
 * @param $op
 *   The operation being performed. One of 'view', 'update', 'create' or
 *   'delete'.
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $account
 *   The user to check for. Leave it to NULL to check for the global user.
 *
 * @return boolean
 *   Whether access is allowed or not. If the entity type does not specify any
 *   access information, NULL is returned.
 */
function entity_admin_check_access($op, $entity_type, $entity = NULL, $account = NULL) {
  $info = entity_admin_entity_get_info($entity_type);
  if (!empty($info['access callback']) && function_exists($info['access callback'])) {
    $funct = $info['access callback'];
    return call_user_func($funct, $op, $entity, $account, $entity_type);
  }
  return NULL;
}

/**
 * Determines whether the given user can perform actions on an entity.
 *
 * Comment and taxonomy_term entities don't store the bundle machine name in
 * there own table but in an extra table. For comment entities the bundle is the
 * type colomn from the node table, joined on nid colomns and for taxonomy term
 * entities the bundle is the machine_name colomn from the taxonomy_vocabulary
 * table, joined on vid colomns. External bundles are dectected in the entity_admin
 * infos providing the value $info['entity admin']['external bundle'] = TRUE.
 *
 * @param $query
 *   Query object, of selection from a entity database table.
 * @param $entity_type
 *   The entity type queried.
 * @param $user
 *   Boolean, Optionally if TRUE, join the user table to the query. Default to
 *   FALSE.
 *
 * @return
 *   A string, the "<table alias>.<colomn name>" from the extra table or False
 *   if the extra table was not joined. The <colomn name> is the 'name' of a user
 *   if $user is TRUE or the bundle value
 */
function entity_admin_join_extra_table(SelectQueryInterface $query, $entity_type, $user = FALSE) {
  $info = entity_admin_entity_get_info($entity_type);
  $base_table = $info['base table'];
  $base_join = FALSE;

  switch ($entity_type) {
    case 'taxonomy_term':
      // From TaxonomyTermController::buildQuery().
      $extra_type = 'taxonomy_vocabulary';
      $extra_table_field = 'machine_name';
      $join_field = $base_join = 'vid';
      $extra_field_alias = $info['entity keys']['bundle'];
      break;
    case 'comment':
      // From CommentController::buildQuery().
      $extra_type = 'node';
      $extra_table_field = 'type';
      $join_field = $base_join = 'nid';
      $extra_field_alias = $info['entity keys']['bundle'];
  }
  if ($user && $entity_type != 'user' && in_array('uid', $info['schema_fields_sql']['base table'])) {
    $extra_type = 'user';
    $extra_table_field = 'name';
    $join_field = 'uid';
    $base_join = (!empty($info['entity admin']['properties']['author'])) ? $info['entity admin']['properties']['author'] : FALSE;
    $extra_field_alias = 'username';
  }

  if (!$base_join) {
    return FALSE;
  }
  $base_table = $info['base table'];
  $tables = $query->getTables();
  $joined_table = FALSE;
  $base_table_alias = FALSE;
  $extra_table_info = entity_admin_entity_get_info($extra_type);
  $extra_table = $extra_table_info['base table'];
  foreach ($tables as $alias => $table_info) {
    if ($table_info['table'] == $extra_table) {
      $joined_table = $alias;
    }
    if ($table_info['table'] == $base_table) {
      $base_table_alias = $alias;
    }
  }
  if (!$base_table_alias) {
    // The comment table is not in the query.
    return FALSE;
  }
  if ($joined_table) {
    // The node table exists. return node table alias.
    return "{$joined_table}.{$extra_table_field}"; // ex : 'n.type'
  }
  else {
    // Attach the table to the query.
    $extra_table_alias = $extra_table;
    $query->innerJoin($extra_table, $extra_table_alias, "{$base_table_alias}.{$base_join} = {$extra_table_alias}.{$join_field}");
    $query->addField($extra_table_alias, $extra_table_field, $extra_field_alias);
    return "{$extra_table_alias}.{$extra_table_field}"; // ex : n.type
  }
}

/**
 * Load an entity or a pseudo entity from entity type and ID.
 *
 * Fork of the entity_load() function but use the entity_admin_get_entity_info()
 * instead of entity_get_info. Usefull for loading objects that are not defined
 * as entity, (ex: node_type or views). The function will use the
 * $info['entity admin']['load callback'] function to load such objects.
 *
 * @param $entity_type
 *   The entity type to load, e.g. node or user.
 * @param $entity_id
 *   The entity id, either the numeric id or the entity name.
 *
 * @return
 *   The entity object, or FALSE.
 *
 * @see entity_load()
 */
function entity_admin_entity_load_entity($entity_type, $entity_id) {
  $info = entity_admin_entity_get_info($entity_type);
  $entity_info = entity_get_info($entity_type);
  if (!empty($info['entity admin']['not entity'])) {
    // Need a specific call back
    if (!empty($info['entity admin']['load callback'])) {
      $entity = call_user_func($info['entity admin']['load callback'], $entity_id);
    }
    if (empty($entity)) {
      return FALSE;
    }
  }
  else {
    if (!is_numeric($entity_id) && !empty($entity_info['entity key']['name'])) {
      $entities = entity_load_multiple_by_name($entity_type, array($entity_id));
      $entity = $entities[$entity_id];
    }
    else {
      $entity = entity_load_single($entity_type, $entity_id);
    }
  }
  if (!empty($entity)) {
    return $entity;
  }
  return FALSE;
}

/**
 * Permanently save an entity or a pseudo entity.
 *
 * Fork of the entity_save() function but use the entity_admin_get_entity_info()
 * instead of entity_get_info. Usefull for saving objects that are not defined
 * as entity, (ex: node_type or views). The function will use the
 * $info['save callback'] function to save such objects.
 *
 * @param $entity_type
 *   The type of the entity.
 * @param $entity
 *   The entity to save.
 *
 * @return
 *   For entity types provided by the CRUD API, SAVED_NEW or SAVED_UPDATED is
 *   returned depending on the operation performed. If there is no information
 *   how to save the entity, FALSE is returned.
 */
function entity_admin_entity_save($entity_type, $entity) {
  $info = entity_admin_entity_get_info($entity_type);
  $return = FALSE;
  if (!empty($info['entity admin']['not entity'])) {
    // Need a specific call back
    if (!empty($info['save callback'])) {
      $return = $info['save callback']($entity);
    }
  }
  else {
    $return = entity_save($entity_type, $entity);
  }

  return $return;
}

/**
 * Returns the label of an entity or a pseudo entity.
 *
 * Fork of the entity_label() function but use the entity_admin_get_entity_info()
 * instead of entity_get_info. Usefull for objects that are not defined as
 * entity, (ex: node_type or views).
 *
 * @param $entity_type
 *   The entity type; e.g., 'node' or 'user' or 'node_type'.
 * @param $entity
 *   The entity for which to generate the label.
 *
 * @return
 *   The entity label, or FALSE if not found.
 */
function entity_admin_entity_label($entity_type, $entity) {
  $info = entity_admin_entity_get_info($entity_type);
  $label = FALSE;
  if (!empty($info['entity admin']['not entity'])) {
    // Need a specific call back
    if (!empty($info['entity keys']['label']) && isset($entity->{$info['entity keys']['label']})) {
      $label = $entity->{$info['entity keys']['label']};
    }
  }
  else {
    $label = entity_label($entity_type, $entity);
  }

  if (!$label) {
    $label = t('@entity_type entity with ID @id', array('@entity_type' => $info['label'], '@id' => $entity->{$info['entity keys']['id']}));
  }
  return $label;
}

function entity_admin_entity_path($entity_type, $entity) {
  $info = entity_admin_entity_get_info($entity_type);
  $entity_path = FALSE;
  if (empty($info['entity admin']['not entity'])) {
    $entity_uri = entity_uri($entity_type, $entity);
    if (is_array($entity_uri) && !empty($entity_uri['path'])) {
      $entity_path = $entity_uri['path'];
    }
  }
  if (!$entity_path) {
    if (!empty($info['bundle of'])) {
      $name_field = (!empty($info['entity keys']['name'])) ? $info['entity keys']['name'] : FALSE;
      $entity_name = ($name_field && isset($entity->$name_field)) ? $entity->$name_field : FALSE;
      $bundle_info = entity_admin_entity_get_info($info['bundle of']);
      foreach ($bundle_info['bundles'] as $bundle_name => $bundle_datas) {
        if ($bundle_name == $entity_name) {
          $entity_label = $bundle_datas['label'];
          if (!empty($bundle_datas['admin']['real path'])) {
            $entity_path = $bundle_datas['admin']['real path'];
          }
          elseif (!empty($bundle_datas['admin']['path'])) {
            $path_array = explode('/', $bundle_datas['admin']['path']);
            $path_array[$bundle_datas['admin']['bundle argument']] = $entity_name;
            $entity_path = implode('/', $path_array);
          }
          break;
        }
      }
    }
    if (!$entity_path) {
      // Fix for complex path.
      if ($entity_type == 'commerce_line_item') {
        // Secial case for the commerce_line_item. it has no view page and no
        // edit page. Use the order page instead.
        if (!empty($entity->order_id)) {
          $entity_path = 'admin/commerce/orders/' . $entity->order_id;
        }
      }
      elseif ($entity_type == 'commerce_payment_transaction') {
        if (!empty($entity->order_id)) {
          $entity_path = 'admin/commerce/orders/' . $entity->order_id . '/payment';
        }
      }
      elseif ($entity_type == 'og_membership') {
        // redirect to the user page
        if ($group = entity_load_single($entity->entity_type, $entity->etid)) {
          if ($uri = entity_uri($entity->entity_type, $group)) {
            $entity_path = $uri['path'];
          }
        }
      }
      elseif (!empty($info['entity admin']['path'])) {
        $id_field = $info['entity keys']['id'];
        $name_field = (!empty($info['entity keys']['name'])) ? $info['entity keys']['name'] : FALSE;
        $entity_id = ($name_field && isset($entity->$name_field)) ? $entity->$name_field : $entity->$id_field;
        $entity_path = $info['entity admin']['path'] . '/' . $entity_id;
      }
    }
  }
  return $entity_path;
}

/**
 * Determines whether the given user can perform actions on an pseudo entity.
 *
 * Used by object that are not defined as defined as entity ('node_type',
 * 'views'). Check for the permission to administer all pseudo entities, defined
 * in $info['entity admin']['administer argument'].
 *
 * @param $op
 *   The operation being performed. One of 'view', 'update', 'create' or
 *   'delete'.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $account
 *   The user to check for. Leave it to NULL to check for the global user.
 * @param $entity_type
 *   The entity type of the entity to check for.
 *
 * @return boolean
 *   Whether access is allowed or not. If the entity type does not specify any
 *   access information, NULL is returned.
 */
function entity_admin_no_entity_access($op, $entity = NULL, $account = NULL, $entity_type) {
  $info = entity_admin_entity_get_info($entity_type);
  if (!empty($info['entity admin']['administer argument'])) {
    // 'administer content types', 'administer views'.
    return (user_access($info['entity admin']['administer argument']));
  }
}

/**
 * Checks if a given entity has a certain exportable status.
 *
 * Fork of the entity_has_status() function but use the
 * entity_admin_get_entity_info() instead of entity_get_info. Usefull for
 * objects that are not defined as entity, (ex: node_type or views).
 *
 * @param $entity_type
 *   The type of the entity.
 * @param $entity
 *   The entity to check the status on.
 * @param $status
 *   The constant status like ENTITY_CUSTOM, ENTITY_IN_CODE, ENTITY_OVERRIDDEN
 *   or ENTITY_FIXED.
 *
 * @return
 *   TRUE if the entity has the status, FALSE otherwise.
 */
function entity_admin_entity_has_status($entity_type, $entity, $status) {
  $info = entity_admin_entity_get_info($entity_type);
  $status_key = empty($info['entity keys']['status']) ? 'status' : $info['entity keys']['status'];
  return isset($entity->{$status_key}) && ($entity->{$status_key} & $status) == $status;
}

/**
 * return the label 2 categories of entities : configurations or datas.
 *
 * Used on displaying entity types on filter form.
 *
 * @param $entity_type
 *   The type of the entity.
 *
 * @return
 *   String the entity category label
 */
function entity_admin_entity_category($entity_type) {
  $info = entity_admin_entity_get_info($entity_type);
  $level1 = (!empty($info['entity admin']['config entity'])) ? t('Configuration') : t('Datas');
  $level1 = '----' . $level1 . '----';
  return $level1;
}

/**
 * Informs if an entity type needs to display languages of entities.
 *
 * If the entity type store language value, look for multiple language entities
 * or translation feature in order to decide if it worths displaying languages.
 *
 * @param $entity_type
 *   The type of the entity.
 *
 * @return
 *   Boolean, TRUE if the language header should be displayed
 */
function entity_admin_display_language($entity_type) {
  $info = entity_admin_entity_get_info($entity_type);
  $lang_content = FALSE;
  if (!empty($info['entity admin']['properties']['language']) && in_array($info['entity admin']['properties']['language'], $info['schema_fields_sql']['base table'])) {
    $lang_col = $info['entity admin']['properties']['language'];
    $lang_content = db_query_range('SELECT 1 FROM {' . $info['base table'] . '} WHERE ' . $lang_col . ' <> :language', 0, 1, array(':language' => LANGUAGE_NONE))->fetchField();
  }
  $multilanguage = (($entity_type == 'node' && module_exists('translation') || (!empty($info['translation']['entity_translation']) && module_exists('entity translation'))));
  return ($lang_content || $multilanguage);
}

/**
 * Implements hook_entity_admin_operations().
 */
function entity_admin_entity_admin_operations() {

  $entity_types = entity_admin_filter_entity_types();

  // Set for default order on keys.
  $operations = array(
    'publish' => FALSE,
    'unpublish' => FALSE,
    'promote' => FALSE,
    'demote' => FALSE,
    'sticky' => FALSE,
    'unsticky' => FALSE,
    'delete' => FALSE,
  );

  foreach ($entity_types as $entity_type => $info) {

    if (!empty($info['entity admin']['properties']['status'])) {

      $operations['publish'] = array(
        'label' => t('Active selected content'),
        'callback' => 'entity_admin_mass_update',
        'callback arguments' => array('updates' => array('status' => NODE_PUBLISHED)),
      );

      $operations['unpublish'] = array(
        'label' => t('Unactive selected content'),
        'callback' => 'entity_admin_mass_update',
        'callback arguments' => array('updates' => array('status' => NODE_NOT_PUBLISHED)),
      );
    }

    if (in_array('promote', $info['schema_fields_sql']['base table'])) {
      $operations['promote'] = array(
        'label' => t('Promote selected @label to front page', array('@label' => $info['label'])),
        'callback' => 'entity_admin_mass_update',
        'callback arguments' => array('updates' => array(
            'status' => NODE_PUBLISHED,
            'promote' => NODE_PROMOTED,
          )),
      );

      $operations['demote'] = array(
        'label' => t('Demote selected @label from front page', array('@label' => $info['label'])),
        'callback' => 'entity_admin_mass_update',
        'callback arguments' => array('updates' => array('promote' => NODE_NOT_PROMOTED)),
      );
    }
    if (in_array('sticky', $info['schema_fields_sql']['base table'])) {
      $operations['sticky'] = array(
        'label' => t('Make selected @label sticky', array('@label' => $info['label'])),
        'callback' => 'entity_admin_mass_update',
        'callback arguments' => array('updates' => array(
            'status' => NODE_PUBLISHED,
            'sticky' => NODE_STICKY,
          )),
      );

      $operations['unsticky'] = array(
        'label' => t('Make selected @label not sticky', array('@label' => $info['label'])),
        'callback' => 'entity_admin_mass_update',
        'callback arguments' => array('updates' => array('sticky' => NODE_NOT_STICKY)),
      );
    }

    $operations['delete'] = array(
      'label' => t('Delete selected content'),
      'callback' => NULL,
    );

  }

  // Delete unused operations.
  foreach ($operations as $name => $data) {
    if (!$data) {
      unset($operations[$name]);
    }
  }

  return $operations;
}

/**
 * List entity administration filters that can be applied.
 *
 * Inspired by the node_filters() core function.
 *
 * @param $entity_type
 *   String, the type of the entity.
 *
 * @return
 *   An associative array of filters.
 */
function entity_admin_filters() {

  // Below : All available filter, set here for ordering.
  $filters = array(
    'entity_type' => FALSE,
    'history' => FALSE,
    'status' => FALSE,
    'language' => FALSE,
    'author' => FALSE,
    'title' => FALSE,
  );
  // Below, default order for status.
  $status_default_order = array(
    '[any]',
    'status-1',
    'status-0',
    'promote-1',
    'promote-0',
    'sticky-1',
    'sticky-0',
    'translate-0',
    'translate-1',
  );
  $entity_types = entity_admin_filter_entity_types();

  $filters['entity_type'] = array(
    'title' => t('Entity type'),
    'options' => array(
      '[any]' => t('any'),
    ),
  );
  $default_types = variable_get('entity_admin_types', array());
  // Filter 'or' for entity types. from all availables entity types.
  foreach (entity_admin_filter_entity_types(FALSE) as $entity_type => $info) {
    if (!empty($default_types) && !in_array($entity_type, $default_types)) {
      continue;
    }
    $level1 = entity_admin_entity_category($entity_type);
    if (!empty($info['entity admin']['properties']['bundle'])) {
      $label_all = t('All @types', array('@types' => $info['plural label']));
      $filters['entity_type']['options'][$level1][$info['label']]["{$entity_type}-[all]"] = $label_all;
      foreach ($info['bundles'] as $bundle_name => $bundle_datas) {
        $filters['entity_type']['options'][$level1][$info['label']]["{$entity_type}-{$bundle_name}"] = $bundle_datas['label'];
      }
    }
    else {
      $filters['entity_type']['options'][$level1][$entity_type] = $info['label'];
    }
  }

  $languages = module_invoke('locale', 'language_list');
  // Build filter on displayed entity types only.
  foreach ($entity_types as $entity_type => $info) {
    global $user;
    if ($user->uid && !empty($info['entity admin']['properties']['history'])) {
      $filters['history'] = array(
        'title' => t('my history'),
        'options' => array(
          '[any]' => t('any'),
          'recent' => t('updated less than one month ago'),
          'new' => t('new content'),
          'updated' => t('changed data'),
        ),
      );
    }

    $has_active = (!empty($info['entity admin']['properties']['status']));
    $has_promote = in_array('promote', $info['schema_fields_sql']['base table']);
    $has_sticky = in_array('sticky', $info['schema_fields_sql']['base table']);
    $has_translate = in_array('translate', $info['schema_fields_sql']['base table']);
    // Regular filters
    if ($has_active || $has_promote || $has_sticky || $has_translate) {
      if (!isset($filters['status']['title'])) {
        $filters['status'] = array(
          'title' => t('status'),
          'options' => array(
            '[any]' => t('any'),
          ),
        );
      }

      if ($has_active) {
        // For views,
        $filters['status']['options']["status-1"] = t('Actived');
        $filters['status']['options']["status-0"] = t('not actived');
      }
      if ($has_promote) {
        $filters['status']['options']['promote-1'] = t('promoted');
        $filters['status']['options']['promote-0'] = t('not promoted');
      }
      if ($has_sticky) {
        $filters['status']['options']['sticky-1'] = t('sticky');
        $filters['status']['options']['sticky-0'] = t('not sticky');
      }
      if ($has_translate) {
        $filters['status']['options'] += array(
          'translate-0' => t('Up to date translation'),
          'translate-1' => t('Outdated translation'),
        );
      }
    }

    // Language filter if there is a list of languages
    $has_language = (!empty($info['entity admin']['properties']['language']));
    $display_language = entity_admin_display_language($entity_type);
    // @TODO : check if there are multilingual languages in the existing entities.
    if ($has_language && $display_language && $languages) {
      $languages = array(LANGUAGE_NONE => t('Language neutral')) + $languages;
      $filters['language'] = array(
        'title' => t('language'),
        'options' => array(
          '[any]' => t('any'),
        ) + $languages,
      );
    }

    $has_author = (!empty($info['entity admin']['properties']['author']));
    if ($has_author) {
      $filters['author'] = array(
        'title' => t('author'),
      );
    }

    $has_title = (!empty($info['entity admin']['properties']['title']));
    if ($has_title) {
      $filters['title'] = array(
        'title' => t('title'),
      );
    }
  }

  // Delete unsed filters
  foreach ($filters as $name => $datas) {
    if (!$datas) {
      unset($filters[$name]);
    }
  }

  // Reorder status.
  if (!empty($filters['status']['options'])) {

    $ordered_status = array();
    foreach ($status_default_order as $name) {
      if (isset($filters['status']['options'][$name])) {
        $ordered_status[$name] = $filters['status']['options'][$name];
      }
    }
    $filters['status']['options'] = $ordered_status;
  }

  return $filters;
}

/**
 * Returns the entity types to display according to options and user filter.
 *
 * @return
 *   An array on admin_entity infos keyed by entity type.
 */
function entity_admin_filter_entity_types($user_filter = TRUE) {
  // Retreive and format filters.
  $filters = entity_admin_format_filters();

  // $entity_types stores the types to show on this page.
  $entity_types = entity_admin_entity_get_info();
  $types_to_show = array();
  $default_types = variable_get('entity_admin_types', array());
  foreach ($entity_types as $type => $info) {
    // Filter entity types.
    // By configuration :
    if (!empty($default_types) && !in_array($type, $default_types)) {
      continue;
    }
    // By user filter on entity type :
    if ($user_filter) {
      $level1 = entity_admin_entity_category($type);
      if (!empty($filters['entity_type']) && empty($filters['entity_type'][$type])) {
        continue;
      }
    }

    // retrieve the corresponding entities.
    // Check about access.
    if (!entity_admin_check_access('view', $type, NULL) || empty($info['entity keys']['id'])) {
      continue;
    }

    // Depending on filter, don't display entity types.
    if ($user_filter) {
      $continue = FALSE;
      foreach (array('language', 'status', 'author', 'title') as $name) {
        if (isset($filters[$name])) {
          $col = (!empty($info['entity admin']['properties'][$name])) ? $info['entity admin']['properties'][$name] : FALSE;
          if (!$col) {
            $continue = TRUE;
            break;
          }
        }
      }
      // More filters.
      // Depending on filter, don't display entity types.
      foreach (array('promote', 'sticky', 'translate') as $name) {
        if (isset($filters[$name]) && !in_array($name, $info['schema_fields_sql']['base table'])) {
          $continue = TRUE;
          break;
        }
      }
      if ($continue) {
        continue;
      }

    }


    // At last, register the entity types that have passed filters.
    $types_to_show[$type] = $info;
  }

  return $types_to_show;
}

/**
 * Returns the formated filter datas from connected user filtering.
 *
 * @return
 *   An array of key : the filter name and value, the value to filter. Keys can
 *   be ''entity_type', 'author', 'status', 'language', promote', sticky',
 *   'history'.
 */
function entity_admin_format_filters() {
  $filters = array();
  $session_name = "entity_admin_overview_filter";
  $filter_data = isset($_SESSION[$session_name]) ? $_SESSION[$session_name] : array();

  foreach ($filter_data as $index => $filter) {
    list($key, $value) = $filter;
    switch ($key) {
      case 'entity_type':
        // Note: no exploitable hole as $key/$value have already been checked when submitted

        // Returns $filter['entity_type'][<entity_type>][<index>] = <bundle>, or
        // $filter['entity_type'][<entity_type>] = TRUE.
        if (strpos($value, '-') !== FALSE) {
          list($entity_type, $value) = explode('-', $value, 2);
          if ($value == '[all]') {
            $filters[$key][$entity_type] = TRUE;
          }
          else {
            $filters[$key][$entity_type][] = $value;
          }
        }
        else {
          $filters[$key][$value] = TRUE;
        }
        break;
      case 'author':
        $filters['author'] = $value;
        break;

      case 'status':
        // $value may be 'translate-0'.
        list($key, $value) = explode('-', $value, 2);
      case 'language':
      case 'history':
      case 'title':
        $filters[$key] = $value;
        break;
    }
  }

  return $filters;
}

/**
 * Returns the entity administration filters form array to entity_admin_admin_entitie().
 *
 * Inspired by the node_filter_form() core function.
 *
 * @see entity_admin_admin_entities()
 * @see entity_admin_admin_entities_submit()
 * @see entity_admin_admin_entities_validate()
 * @see entity_admin_filter_form_submit()
 * @see entity_admin_multiple_delete_confirm()
 * @see entity_admin_multiple_delete_confirm_submit()
 *
 * @ingroup forms
 */
function entity_admin_filter_form() {

  $session_name = "entity_admin_overview_filter";
  $session = isset($_SESSION[$session_name]) ? $_SESSION[$session_name] : array();
  $filters = entity_admin_filters();

  if (empty($filters)) {
    return array();
  }
  $i = 0;
  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Show only items where'),
    '#theme' => 'exposed_filters__node',
  );
  $entity_types = entity_admin_entity_get_info();
  foreach ($session as $filter) {
    list($type, $value) = $filter;
    $is = t('is');
    if ($type == 'language') {
      $value = $value == LANGUAGE_NONE ? t('Language neutral') : module_invoke('locale', 'language_name', $value);
    }
    elseif ($type == 'entity_type') {
      // Handle 2 levels selectbox.
      if (strpos($value, '-') !== FALSE) {
        // This entity owns bundles.
        list($entity_type, $bundle) = explode('-', $value, 2);
        // Retreive entity type label
        $info = $entity_types[$entity_type];
        $entity_type_label = $info['label'];
        $level1 = entity_admin_entity_category($entity_type);

        if ($bundle == '[all]') {
          // All entities are required.
          // Returns text for the filter status
          $value = $entity_type_label;
          // Delete the all entity entry
          unset($filters['entity_type']['options'][$level1][$entity_type_label]);
        }
        else {
          // A specific bundle is required.
          // Save text for the filter status.
          $bundle_label = $filters['entity_type']['options'][$level1][$entity_type_label][$value];
          // Delete 'all' and the bundle.
          unset($filters['entity_type']['options'][$level1][$entity_type_label][$value]);
          if (isset($filters['entity_type']['options'][$level1][$entity_type_label]["{$entity_type}-[all]"])) {
            unset($filters['entity_type']['options'][$level1][$entity_type_label]["{$entity_type}-[all]"]);
          }
          if (isset($filters['entity_type']['options'][$level1][$entity_type_label]) && empty($filters['entity_type']['options'][$level1][$entity_type_label])) {
            // There is no bundle left. Delete the entity type entry.
            unset($filters['entity_type']['options'][$level1][$entity_type_label]);
          }
          // Returns text for the filter status.
          $value = t('@entity_type of type @bundle', array('@entity_type' => $entity_type_label, '@bundle' => $bundle_label));
        }

      }
      else {
        // This entity does not own bundles.
        $entity_type = $value;
        $info_type = $entity_types[$entity_type];
        $level1 = entity_admin_entity_category($entity_type);
        $label = $filters['entity_type']['options'][$level1][$value];
        // Note : the options is already deleted in
        // entity_admin_filter_entity_types().
        unset($filters['entity_type']['options'][$level1][$value]);
        // return text for displaying filter status.
        $value = $label;
      }
    }
    elseif ($type == 'author') {
      $account = user_load($value);
      $value = $account->name;
    }
    elseif (isset($filters[$type]['options'][$value])) {
      // Save the label.
      $label = $filters[$type]['options'][$value];
      unset($filters[$type]['options'][$value]);
      $value = $label;
    }
    else {
      // $type == 'title'
      // nothing to do.
      $is = t('contains');
    }
    $t_args = array(
      '%property' => $filters[$type]['title'],
      '%value' => $value,
      '@is' => $is,
    );
    if ($i++) {
      $form['filters']['current'][] = array('#markup' => t('and where %property @is %value', $t_args));
    }
    else {
      $form['filters']['current'][] = array('#markup' => t('where %property @is %value', $t_args));
    }
    if (in_array($type, array('language', 'author', 'history', 'title'))) {
      // Remove the option if it is already being filtered on.
      unset($filters[$type]);
    }
  }

  $form['filters']['status'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('clearfix')),
    '#prefix' => ($i ? '<div class="additional-filters">' . t('and where') . '</div>' : ''),
  );
  $form['filters']['status']['filters'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('filters')),
  );
  foreach ($filters as $key => $filter) {
    if (isset($filter['options'])) {
      $form['filters']['status']['filters'][$key] = array(
        '#type' => 'select',
        '#options' => $filter['options'],
        '#title' => $filter['title'],
        '#default_value' => '[any]',
      );
    }
    elseif ($key == 'author') {
      // Author filter.
      $form['filters']['status']['filters'][$key] = array(
        '#type' => 'textfield',
        '#title' => $filter['title'],
        '#default_value' => '',
        '#maxlength' => 60,
        '#size' => 25,
        '#autocomplete_path' => 'user/autocomplete',
        '#access' => user_access('access user profiles'),
      );
    }
    elseif ($key == 'title') {
      // Title filter.
      $form['filters']['status']['filters'][$key] = array(
        '#type' => 'textfield',
        '#title' => $filter['title'],
        '#default_value' => '',
        '#maxlength' => 60,
        '#size' => 25,
      );
    }
  }

  $form['filters']['status']['actions'] = array(
    '#type' => 'actions',
    '#attributes' => array('class' => array('container-inline')),
  );
  $form['filters']['status']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => count($session) ? t('Refine') : t('Filter'),
  );
  if (count($session)) {
    $form['filters']['status']['actions']['undo'] = array(
      '#type' => 'submit',
      '#value' => t('Undo'),
    );
    $form['filters']['status']['actions']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset'),
    );
  }

  drupal_add_js('misc/form.js');

  return $form;
}

/**
 * Form submission handler for entity_admin_filter_form().
 *
 * Inspired by the node_filter_form_submit() core function.
 *
 * @see entity_admin_admin_content()
 * @see entity_admin_admin_entities()
 * @see entity_admin_admin_entities_submit()
 * @see entity_admin_admin_entities_validate()
 * @see entity_admin_filter_form()
 * @see entity_admin_multiple_delete_confirm()
 * @see entity_admin_multiple_delete_confirm_submit()
 */
function entity_admin_filter_form_submit($form, &$form_state) {

  $session_name = "entity_admin_overview_filter";
  $filters = entity_admin_filters();
  switch ($form_state['values']['op']) {
    case t('Filter'):
    case t('Refine'):
      // Apply every filter that has a choice selected other than 'any'.
      foreach ($filters as $filter => $options) {
        if (isset($form_state['values'][$filter]) && $form_state['values'][$filter] != '[any]') {
          if (isset($filters[$filter]['options'])) {
            // Flatten the options array to accommodate hierarchical/nested options.
            $flat_options = form_options_flatten($filters[$filter]['options']);
            // Only accept valid selections offered on the dropdown, block bad input.
            if (isset($flat_options[$form_state['values'][$filter]])) {
              $_SESSION[$session_name][] = array($filter, $form_state['values'][$filter]);
            }
          }
          elseif ($filter == 'author' && isset($form_state['values'][$filter]) && $account = user_load_by_name($form_state['values'][$filter])) {
            // Do not load anonymous user
            if (!empty($account->uid)) {
              $_SESSION[$session_name][] = array($filter, $account->uid);
            }
          }
          elseif ($filter == 'title' && $form_state['values'][$filter] !== '') {
            $_SESSION[$session_name][] = array($filter, $form_state['values'][$filter]);
          }
        }
      }
      break;
    case t('Undo'):
      array_pop($_SESSION[$session_name]);
      break;
    case t('Reset'):
      $_SESSION[$session_name] = array();
      break;
  }
}

/**
 * Make mass update of entities, changing all entities in the $entities array
 * to update them with the field values in $updates.
 *
 * IMPORTANT NOTE: This function is intended to work when called from a form
 * submission handler. Calling it outside of the form submission process may not
 * work correctly. Inspired by the node_mass_update() core function.
 *
 * @param array $entities
 *   Array of entities ids to update.
 * @param array $updates
 *   Array of key/value pairs with entity field names and the value to update
 *   that field to.
 */
function entity_admin_mass_update($entities, $updates) {
  // We use batch processing to prevent timeout when updating a large number
  // of entities.
  if (count($entities) > 10) {
    $batch = array(
      'operations' => array(
        array('_entity_admin_mass_update_batch_process', array($entities, $updates)),
      ),
      'finished' => '_entity_admin_mass_update_batch_finished',
      'title' => t('Processing'),
      // We use a single multi-pass operation, so the default
      // 'Remaining x of y operations' message will be confusing here.
      'progress_message' => '',
      'error_message' => t('The update has encountered an error.'),
      // The operations do not live in the .module file, so we need to
      // tell the batch engine which file to load before calling them.
      'file' => drupal_get_path('module', 'entity_admin') . '/entity_admin_page.inc',
    );
    batch_set($batch);
  }
  else {
    foreach ($entities as $id) {
      _entity_admin_mass_update_helper($id, $updates);
    }
    drupal_set_message(t('The update has been performed.'));
  }
}

/**
 * Updates individual entities when fewer than 10 are queued.
 *
 * @param $entity_type
 *   String, the type of the entity.
 * @param $id
 *   ID of entity to update.
 * @param $updates
 *   Associative array of updates.
 *
 * @return object
 *   An updated entity object.
 *
 * @see entity_admin_mass_update()
 */
function _entity_admin_mass_update_helper($id, $updates) {
  list($entity_type, $entity_id) = explode('-', $id, 2);
  $entity = entity_admin_entity_load_entity($entity_type, $entity_id);
  // For efficiency manually save the original entity before applying any changes.
  $entity->original = clone $entity;
  foreach ($updates as $name => $value) {
    // Handle status col name.
    if ($name == 'status') {
      $info = entity_admin_entity_get_info($entity_type);
      $status_col = (!empty($info['entity admin']['properties']['status'])) ? $info['entity admin']['properties']['status'] : FALSE;
      if (is_string($status_col) && $status_col == 'disabled' || $entity_type == 'views') {
        // For views, $status_col is TRUE because it is not stored in the db.
        $name = 'disabled';
        $value = ($value == 0) ? 1 : 0;
      }
      elseif (is_string($status_col) && $status_col != 'status') {
        // exemple : 'active' for rules_config.
        $name = $status_col;
      }
    }
    $entity->$name = $value;
  }

  entity_admin_entity_save($entity_type, $entity);

  if ($entity_type == 'views' && $name == 'disabled') {
    // Views disabling/enabling if not simply update the view object.
    //$views_name = $entity->{$info['entity keys']['name']};
    $value = (bool) $value;
    ctools_export_crud_set_status("views_view", $entity, $value);
  }
  return $entity;
}

/**
 * Executes a batch operation for entity_admin_mass_update().
 *
 * Inspired by the node_mass_update_batch_process() core function.
 *
 * @param array $entities
 *   An array of entities IDs.
 * @param array $updates
 *   Associative array of updates.
 * @param array $context
 *   An array of contextual key/values.
 */
function _entity_admin_mass_update_batch_process($entities, $updates, &$context) {
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = count($entities);
    $context['sandbox']['entities'] = $entities;
  }

  // Process entities by groups of 5.
  $count = min(5, count($context['sandbox']['entities']));
  for ($i = 1; $i <= $count; $i++) {
    // For each id, load the entities, reset the values, and save it.
    $id = array_shift($context['sandbox']['entities']);
    $entity = _entity_admin_mass_update_helper($id, $updates);

    list($entity_type, $entity_id) = explode('-', $id, 2);
    // Store result for post-processing in the finished callback.
    $label = entity_admin_entity_label($entity_type, $entity);
    $path = entity_admin_entity_path($entity_type, $entity);
    $display = (!empty($path)) ? l($label, $path) : $label;
    $context['results'][] = $display;

    // Update our progress information.
    $context['sandbox']['progress']++;
  }

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
}

/**
 * Menu callback: Reports the status of batch operation for entity_admin_mass_update().
 *
 * Inspired by the node_mass_update_batch_finished() core function.
 *
 * @param bool $success
 *   A boolean indicating whether the batch mass update operation successfully
 *   concluded.
 * @param int $results
 *   The number of entities updated via the batch mode process.
 * @param array $operations
 *   An array of function calls (not used in this function).
 */
function _entity_admin_mass_update_batch_finished($success, $results, $operations) {
  if ($success) {
    drupal_set_message(t('The update has been performed.'));
  }
  else {
    drupal_set_message(t('An error occurred and processing did not complete.'), 'error');
    $message = format_plural(count($results), '1 item successfully processed:', '@count items successfully processed:');
    $message .= theme('item_list', array('items' => $results));
    drupal_set_message($message);
  }
}

/**
 * Page callback: Form constructor for the entity administration form.
 *
 * Inspired by the node_admin_content() core function.
 *
 * @see entity_admin_admin_entities()
 * @see entity_admin_admin_entities_submit()
 * @see entity_admin_admin_entities_validate()
 * @see entity_admin_filter_form()
 * @see entity_admin_filter_form_submit()
 * @see entity_menu()
 * @see entity_admin_multiple_delete_confirm()
 * @see entity_admin_multiple_delete_confirm_submit()
 * @ingroup forms
 */
function entity_admin_admin_content($form, $form_state) {
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return entity_admin_multiple_delete_confirm($form, $form_state, array_filter($form_state['values']['entities']));
  }

  $form['filter'] = entity_admin_filter_form();
  $form['#submit'][] = 'entity_admin_filter_form_submit';
  $form['admin'] = entity_admin_admin_entities();

  return $form;
}

/**
 * Form builder: Builds the entity administration overview.
 *
 * Inspired by the node_admin_nodes() core function.
 *
 * @see entity_admin_admin_entities_submit()
 * @see entity_admin_admin_entities_validate()
 * @see entity_admin_filter_form()
 * @see entity_admin_filter_form_submit()
 * @see entity_admin_multiple_delete_confirm()
 * @see entity_admin_multiple_delete_confirm_submit()
 *
 * @ingroup forms
 */
function entity_admin_admin_entities() {

  $admin_access = user_access('administer entity admin overview');

  // Build the 'Update options' form.
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#attributes' => array('class' => array('container-inline')),
    '#access' => $admin_access,
  );
  $options = array();

  foreach (module_invoke_all('entity_admin_operations') as $operation => $array) {
    if (!empty($array['label'])) {
      $options[$operation] = $array['label'];
    }
  }

  $form['options']['operation'] = array(
    '#type' => 'select',
    '#title' => t('Operation'),
    '#title_display' => 'invisible',
    '#options' => $options,
    '#default_value' => 'approve',
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
    '#validate' => array('entity_admin_admin_entities_validate'),
    '#submit' => array('entity_admin_admin_entities_submit'),
  );

  // Sort.
  $sort_default = array(
    'order' => 'changed',
    'sort' => 'desc',
  );
  $user_sort = drupal_get_query_parameters();
  if (!empty($user_sort)) {
    // The user provide a custom sort.
    // Sort order is the label name. Retreive the machine-name
    foreach (entity_admin_properties() as $name => $label) {
      if (!empty($user_sort['order']) && $user_sort['order'] == $label) {
        $sort = $user_sort;
        $sort['order'] = $name;
        break;
      }
    }
  }
  if (empty($sort)) {
    $sort = $sort_default;
  }
  if (empty($sort['sort'])) {
    $sort['sort'] = 'ASC';
  }
  $sort['sort'] = drupal_strtoupper($sort['sort']);

  $entity_types = entity_admin_filter_entity_types();
  // Build the sortable table header.
  $all_headers = entity_admin_properties();
  // Set header not for storing the colonm order from config.
  $header = array_flip(array_keys($all_headers));
  $default_headers = variable_get('entity_admin_headers', array());
  foreach ($entity_types as $type => $info) {
    foreach ($all_headers as $name => $title) {
      $exclude_header = !empty($default_headers) && !in_array($name, $default_headers);
      $type_header = (!empty($info['entity admin']['properties'][$name])) ? $info['entity admin']['properties'][$name] : FALSE;
      if (!$exclude_header && $type_header && ($name != 'language' || entity_admin_display_language($type))) {
        if ($name == 'language' && !(entity_admin_display_language($type))) {
          continue;
        }
        $header[$name] = array('data' => $title);

        $header[$name]['field'] = "$type_header";
        if (is_string($type_header) && $sort['order'] == $name) {
          $header[$name]['sort'] = drupal_strtolower($sort['sort']);
        }

      }
      elseif (!isset($header[$name]['data'])) {
        // Do not display empty headers.
        $header[$name] = '';
      }
    }
  }
  $header['operations'] = array('data' => t('Operations'));
  // Delete unused colomns.
  $header = array_filter($header);

  // Fix the sort if the colomn does not exists for thoses entity types.
  if (!isset($header[$sort['order']])) {
    $sort = $sort_default;
    $header[$sort['order']]['sort'] = drupal_strtolower($sort['sort']);
  }

  // Retreive entities
  $all_entities = array();
  // Retreive and format filters.
  $filters = entity_admin_format_filters();

  global $user;

  foreach ($entity_types as $type    => $info) {
    // retrieve the corresponding entities.

    $alias = $info['base table'];
    $id_field = $info['entity keys']['id'];
    $ea_table_ent_id = (in_array($info['base table field types'][$id_field], array(
      'int',
      'serial',
    ))) ? 'entity_id' : 'entity_name';

    $select = db_select($info['base table'], $alias);
    $select->addField($alias, $id_field, 'entity_id');
    // Add more fields to avoid to load the entity for title, etc...
    $name_field = (!empty($info['entity keys']['name'])) ? $info['entity keys']['name'] : FALSE;
    if ($name_field) {
      // Used for loading the node_type "entity" and for sorting by human name
      // intead of machine name.
      $select->addField($alias, $name_field, 'name');
    }

    // Apply filters on bundles.
    if (!empty($filters['entity_type'][$type]) && is_array($filters['entity_type'][$type])) {
      foreach ($filters['entity_type'][$type] as $bundle) {
        // Special case for taxonomy_term and comment, Bundle value is not in
        // the base table.
        if (!empty($info['entity admin']['external bundle']) && $loc = entity_admin_join_extra_table($select, $type)) {
          $select->condition($loc, $bundle);
        }
        else {
          $select->condition("{$alias}.{$info['entity keys']['bundle']}", $bundle);
        }
      }
    }

    // Depending on filter, don't display some entities.
    foreach (array('language', 'status', 'author', 'title') as $name) {
      if (!empty($filters[$name]) && !empty($info['entity admin']['properties'][$name]) && is_string($info['entity admin']['properties'][$name])) {
        $operator = '=';
        $value = $filters[$name];
        if ($name == 'status') {
          if ($value == 1) {
            // Satus activated. change '= 1' to '>= 1'.
            $operator = '>=';
          }
          if ($info['entity admin']['properties'][$name] == 'disabled') {
            // Inverse.
            $value = ($value == 1) ? 0 : 1;
            $operator = '=';
          }
        }
        elseif ($name == 'title') {
          $value = "%{$value}%";
          $operator = 'LIKE';
        }
        $select->condition("{$alias}.{$info['entity admin']['properties'][$name]}", $value, $operator);
      }
    }

    $own_changed = (!empty($info['entity admin']['properties']['changed']) && is_string($info['entity admin']['properties']['changed'])) ? $info['entity admin']['properties']['changed'] : FALSE;
    $field_changed = ($own_changed) ? "{$alias}.{$own_changed}" : 'eac.changed';
    // Filter recent history
    if (!empty($filters['history'])) {
      if ($filters['history'] == 'recent') {
        if ($own_created = $info['entity admin']['properties']['created']) {
          $db_or_recent = db_or()
            ->condition("{$alias}.{$own_created}", NODE_NEW_LIMIT, '>')
            ->condition($field_changed, NODE_NEW_LIMIT, '>');
          $select->condition($db_or_recent);
        }
        else {
          // the eac table will be added below.
          // as a left join.
          $select->condition($field_changed, NODE_NEW_LIMIT, '>');
        }
      }
      elseif ($filters['history'] == 'new' && $user->uid) {
        // join the last changed table :Left join.
        $select->leftJoin('entity_admin_history', 'eah', "eah.entity_type = '$type' AND eah.uid = '$user->uid' AND eah.$ea_table_ent_id = {$alias}.$id_field");
        $select->isNull('eah.uid');
        $select->condition($field_changed, NODE_NEW_LIMIT, '>');
      }
      elseif ($filters['history'] == 'updated' && $user->uid) {
        // join the last changed view : Inner join
        $select->innerJoin('entity_admin_history', 'eah', "eah.entity_type = '$type' AND eah.uid = '$user->uid' AND eah.$ea_table_ent_id = {$alias}.$id_field");
        $select->innerJoin('entity_admin_changed', 'eac', "eac.entity_type = '$type' AND eac.$ea_table_ent_id = {$alias}.$id_field AND $field_changed > eah.timestamp");
      }
    }

    // More filters, based on colomn name.
    foreach (array('promote', 'sticky', 'translate') as $name) {
      if (isset($filters[$name]) && in_array($name, $info['schema_fields_sql']['base table'])) {
        $select->condition("{$alias}.{$name}", $filters[$name]);
      }
    }

    // Special case for nodes and comments : display entities regarding
    // permissions
    if (in_array($type, array('node', 'comment'))) {
      // Set the node table alias.
      if ($type == 'comment') {
        // Join the node table
        $loc_bundle = entity_admin_join_extra_table($select, $type);
        list($node_alias, $bundle_field) = explode('.', $loc_bundle, 2);
        if (!user_access('administer comments') && !user_access('skip comment approval')) {
          $select->condition("{$alias}.status", 1);
        }
      }
      else {
        $node_alias = $alias;
      }
      if (!user_access('bypass node access')) {
        // If the user is able to view their own unpublished nodes, allow them
        // to see these in addition to published nodes. Check that they actually
        // have some unpublished nodes to view before adding the condition.
        if (user_access('view own unpublished content') && $own_unpublished = db_query('SELECT nid FROM {node} WHERE uid = :uid AND status = :status', array(':uid' => $GLOBALS['user']->uid, ':status' => 0))->fetchCol()) {
          $select->condition(db_or()
            ->condition("{$node_alias}.status", 1)
            ->condition("{$node_alias}.nid", $own_unpublished, 'IN')
          );
        }
        else {
          // If not, restrict the query to published nodes.
          $select->condition("{$node_alias}.status", 1);
        }
      }

      $select->addTag('node_access');
    }
    elseif (empty($info['entity admin']['not entity'])) {
      // Add permissions control if defined.
      if (!empty($info['access arguments']['access tag'])) {
        $select->addTag($info['access arguments']['access tag']);
      }
    }

    $primary_sort = (!in_array($sort['order'], array('changed')));
    // Set order. Ad a 'sort' result set, used for php sorting.
    if ($primary_sort) {
      $col = (!empty($info['entity admin']['properties'][$sort['order']])) ? $info['entity admin']['properties'][$sort['order']] : FALSE;
      if ($col && is_string($col)) {
        // Exception for history : it will return TRUE instead of a colomn name.

        // The sort colomn exists on the entity.
        // Provide a 'sort' value in the result.
        // Special case for taxonomy_term and comment : bundle value is not
        // in the base table.
        if ($sort['order'] == 'bundle' && !empty($info['entity admin']['external bundle'])) {
          if ($loc = entity_admin_join_extra_table($select, $type)) {
            list($table_alias, $bundle_field) = explode('.', $loc, 2);
            $select->addField($table_alias, $bundle_field, 'sort');
            $select->orderBy($loc, $sort['sort']);
          }
        }
        elseif ($sort['order'] == 'author') {
          // Join the user table for adding name as sot key to the query results.
          if ($loc = entity_admin_join_extra_table($select, $type, TRUE)) {
            list($table_alias, $extra_field) = explode('.', $loc, 2);
            $select->addField($table_alias, $extra_field, 'sort');
            $select->orderBy($loc, $sort['sort']);
          }
        }
        elseif (in_array($col, $info['schema_fields_sql']['base table'])) {
          $select->addField($alias, $col, 'sort');
          $select->orderBy("{$alias}.{$col}", $sort['sort']);
        }

      }
    }

    // Always provide changed infos for 2nd sort key

    if ($own_changed) {
      $select->addField($alias, $own_changed, 'changed');
      // Provide a 'sort' value in the result.
      if (!$primary_sort) {
        $select->addField($alias, $own_changed, 'sort');
      }
      $select->orderBy($field_changed, 'DESC');
    }
    else {
      // No changed property, use the entity_admin_changed one.
      if (empty($filters['history']) || $filters['history'] != 'updated') {
        $select->leftJoin('entity_admin_changed', 'eac', "eac.entity_type = '$type' AND eac.$ea_table_ent_id = {$alias}.$id_field");
      }
      $select->addField('eac', 'changed', 'changed');
      // Provide a 'sort' value in the result.
      if (!$primary_sort) {
        $select->addField('eac', 'changed', 'sort');
      }
      $select->orderBy("eac.changed", 'DESC');
    }

    $ids = $select->execute()->fetchAllAssoc('entity_id', PDO::FETCH_ASSOC);

    if ($type == 'views') {
      // Load default views, stored in code and not in the database.
      ctools_include('export');
      $all_views = ctools_export_crud_load_all($info['base table']);
      $views_not_db = array();
      $filter_on_status = (isset($filters['status']));
      $filter_on_title = (isset($filters['title']));
      if ($filter_on_title) {
        $pattern = '#' . quotemeta($filters['title']) . '#';
      }
      foreach ($all_views as $views_name => $view) {
        // Apply filters on views.
        $active = (int) (empty($view->disabled));
        $match = TRUE;
        if ($filter_on_title) {
          $view_name = $view->human_name;
          $match = preg_match($pattern, $view_name);
        }
        $exclude_view = (($filter_on_status && $active != $filters['status']) || ($filter_on_title && !$match));
        if ($view->vid == 0 && !$exclude_view) {
          $views_not_db[$view->name]['entity_id'] = $view->name;
          foreach (array('human_name', 'name', 'vid') as $field) {
            $views_not_db[$view->name][$field] = $view->$field;
          }
        }
        if ($exclude_view && $view->vid != 0) {
          unset($ids[$view->vid]);
        }
      }
      if (!empty($views_not_db)) {
        // Retreive changed date for default views.
        $view_changed = db_select('entity_admin_changed', 'eac')
          ->fields('eac', array('changed', 'entity_name'))
          ->condition('entity_type', 'views')
          ->condition('entity_id', 0)
          ->condition('entity_name', array_keys($views_not_db), 'IN')
          ->execute();
        foreach ($view_changed as $row) {
          $views_not_db[$row->name]['changed'] = $row['changed'];
          if (!$primary_sort) {
            $views_not_db[$row->name]['sort'] = $row['changed'];
          }
        }
        $ids += $views_not_db;
      }
    }

    foreach ($ids as $id => $datas) {
      // Do not return the anonymous user.
      if ($type == 'user' && $id == 0) {
        continue;
      }
      $store = array(
        'entity_type' => $type,
        'entity_id' => $id,
      ) + $datas;
      // $store += array('sort' => -10);
      $all_entities[] = $store;
    }
  }

  // Prepare the list of entities.
  // Sort using php.

  $sorted_from_db = $not_sorted_from_db = $sorted_changed = $sorted_from_db_entity = array();
  foreach ($all_entities as $key => $row) {
    if (isset($row['sort']) || $sort['order'] == 'entity_type') {
      // Alter the sort value :
      $sort_value = (isset($row['sort'])) ? $row['sort'] : '';
      if ($sort['order'] == 'bundle') {
        // Sort on the human name instead of the machine name.
        $info = $entity_types[$row['entity_type']];
        if (isset($info['bundles'][$sort_value]['label'])) {
          $sort_value = $info['bundles'][$sort_value]['label'];
        }
      }
      elseif ($sort['order'] == 'entity_type') {
        $info = $entity_types[$row['entity_type']];
        $sort_value = $info['label'];
      }
      // No case sensitive sort.
      $sort_value = drupal_strtolower($sort_value);

      $sorted_from_db[$key] = $sort_value;
      $sorted_changed[$key] = (isset($row['changed'])) ? (int) $row['changed'] : 0;
      $sorted_from_db_entity[$key] = $row;
    }
    else {
      $not_sorted_from_db[$key] = $row;
    }

  }

  // Sort naturel needs at least php 5.4
  $php_sortnat = (phpversion() >= 5.4) ? SORT_NATURAL : SORT_ASC;
  $php_sort = ($sort['sort'] != 'DESC') ? $php_sortnat : SORT_DESC;
  array_multisort($sorted_from_db, $php_sort, $sorted_changed, SORT_DESC, $sorted_from_db_entity);
  $sorted_entities = $sorted_from_db_entity;
  $sorted_entities += $not_sorted_from_db;
  $all_entities = $sorted_entities;


  // Now all entities are sorted.
  // add pager.
  $per_page = variable_get('entity_admin_entities_per_page', 50);
  // Initialise the pager
  $current_page = pager_default_initialize(count($all_entities), $per_page);
  // Split your list into page sized chunks
  $chunks = array_chunk($all_entities, $per_page, TRUE);
  $page = pager_find_page();
  $offset = $per_page * $page;
  // If no result or page does not exists.
  if (!isset($chunks[$page])) {
    $page = 0;
    if (!isset($chunks[$page])) {
      $chunks[$page] = array();
    }
  }

  $destination = drupal_get_destination();
  $options = array();
  foreach ($chunks[$page] as $key => $stored_datas) {
    // Display entities on page.

    $entity_type = $stored_datas['entity_type'];
    $entity_id = $stored_datas['entity_id'];

    $info = $entity_types[$entity_type];

    if (!empty($info['entity admin']['not entity'])) {
      // for node type, and views, need the name value.
      // Infos on taxonomy_vocabulary does not provide entity keys "name",
      // loading this entity by name will fail.
      if ($entity_type != 'taxonomy_vocabulary' && !empty($stored_datas['name'])) {
        $entity_id = $stored_datas['name'];
      }
    }
    $entity = entity_admin_entity_load_entity($entity_type, $entity_id);
    if (!$entity) {
      // The entity could not be loaded. Skip it and display a message
      drupal_set_message(t('@entity_type entity with ID @id could not be loaded.', array('@entity_type' => $info['label'], '@id' => $entity_id)), 'warning');
      continue;
    }
    $entity_ref = "{$entity_type}-{$entity_id}";

    foreach ($header as $name => $data) {
      if ($name == 'operations') {
        continue;
      }
      $options[$entity_ref][$name] = entity_admin_entity_row($entity_type, $entity, $name, $stored_datas);
    }
    $operations = array();
    $entity_path = entity_admin_entity_path($entity_type, $entity);
    $default_links = variable_get('entity_admin_links', array());
    foreach (entity_admin_page_link_operations() as $name => $title) {
      $exclude_link = !empty($default_links) && !in_array($name, $default_links);
      if (!$exclude_link && $op_path = entity_admin_page_links($entity_type, $entity, $name, $entity_path)) {
        $operations[] = array(
          'title' => $title,
          'href' => $op_path,
          'query' => $destination,
        );
      }
    }

    if (count($operations) > 1) {
      // Render an unordered list of operations links.
      $options[$entity_ref]['operations'] = array(
        'data' => array(
          '#theme' => 'links__node_operations',
          '#links' => $operations,
          '#attributes' => array('class' => array('links', 'inline')),
        ),
      );
    }
    elseif (!empty($operations)) {
      // Render the first and only operation as a link.
      $link = reset($operations);
      $options[$entity_ref]['operations'] = array(
        'data' => array(
          '#type' => 'link',
          '#title' => $link['title'],
          '#href' => $link['href'],
          '#options' => array('query' => $link['query']),
        ),
      );
    }
    else {
      $options[$entity_ref]['operations'] = ''; // empty cell.
    }
  }

  // Add a summary.
  $count_results = count($all_entities);
  $form['summary'] = array(
    '#markup' => ($count_results) ? format_plural(count($all_entities), 'There is 1 result.', 'There are @count results.') : t('No results'),
    '#prefix' => '<strong>',
    '#suffix' => '</strong>',
  );

  // Only use a tableselect when the current user is able to perform any
  // operations.
  if ($admin_access) {
    $form['entities'] = array(
      '#type' => 'tableselect',
      '#header' => $header,
      '#options' => $options,
      '#empty' => t('No content available.'),
    );
  }
  // Otherwise, use a simple table.
  else {
    $form['entities'] = array(
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $options,
      '#empty' => t('No content available.'),
    );
  }

  $form['pager'] = array('#markup' => theme('pager'));
  return $form;
}

/**
 * Validate entity_admin_admin_entities form submissions.
 *
 * Checks whether any entities have been selected to perform the chosen 'Update
 * option' on. Inspired by the node_admin_nodes_validate() core function.
 *
 * @see entity_admin_admin_entities()
 * @see entity_admin_admin_entities_submit()
 * @see entity_admin_filter_form()
 * @see entity_admin_filter_form_submit()
 * @see entity_admin_multiple_delete_confirm()
 * @see entity_admin_multiple_delete_confirm_submit()
 */
function entity_admin_admin_entities_validate($form, &$form_state) {
  // Error if there are no items to select.
  if (!is_array($form_state['values']['entities']) || !count(array_filter($form_state['values']['entities']))) {
    form_set_error('', t('No items selected.'));
    return;
  }

  // Specific validation on entities.
  $entities = array_filter($form_state['values']['entities']);
  $operations = module_invoke_all('entity_admin_operations');
  $operation_name = $form_state['values']['operation'];
  $operation = $operations[$operation_name];

  foreach ($entities as $entity_ref => $datas) {
    list($entity_type, $entity_id) = explode('-', $entity_ref, 2);
    $info = entity_admin_entity_get_info($entity_type);
    $entity = entity_admin_entity_load_entity($entity_type, $entity_id);
    if (!$entity) {
      // The entity could not be loaded. Skip it and display a message
      form_set_error($entity_ref, t('@entity_type entity with ID @id could not be loaded.', array('@entity_type' => $info['label'], '@id' => $entity_id)));
      continue;
    }
    $entity_label = entity_admin_entity_label($entity_type, $entity);
    $args = array(
      '@entity_type' => $info['label'],
      '%title' => $entity_label,
    );

    if ($operation_name == 'delete') {
      if ($entity_type == 'user' && $entity_id == 1) {
        // from user_multiple_cancel_confirm(().
        form_set_error($entity_ref, t('The user account %name cannot be cancelled.', array('%name' => $entity->name)));
      }
      $delete_access = entity_admin_check_access('delete', $entity_type, $entity);
      if ($delete_access === FALSE) {
        form_set_error($entity_ref, t('Your are not allowed to delete the %title @entity_type.', $args));
      }
      elseif ($delete_access === NULL) {
        // Entity access callback is not provided in the entity info...
        form_set_error($entity_ref, t('Delete permission can not be checked for %title @entity_type.', $args));
      }
    }
    elseif ($operation['callback'] == 'entity_admin_mass_update') {
      // This is an update operation.
      $update_access = entity_admin_check_access('update', $entity_type, $entity);
      if ($update_access === FALSE) {
        form_set_error($entity_ref, t('Your are not allowed to update the %title @entity_type.', $args));
      }
      elseif ($update_access === NULL) {
        // Entity access callback is not provided in the entity info...
        form_set_error($entity_ref, t('Update permission can not be checked for %title @entity_type', $args));
      }
      if (!empty($info['entity admin']['not entity']) && (empty($info['save callback']) || !function_exists($info['save callback']))) {
        // Need a specific call back
        form_set_error($entity_ref, t('Update is not available for @entity_type entity types.', $args));
      }
      $updates = (!empty($operation['callback arguments']['updates'])) ? $operation['callback arguments']['updates'] : array();
      $not_available = FALSE;
      foreach ($updates as $col => $new_value) {
        $args_update = $args;
        $args_update['%operation'] = $operation['label'];
        if ($col == 'status') {
          if (empty($info['entity admin']['properties']['status']) || $entity_type == 'node_type') {
            // Node_type should not be disabled from a user interface. it is for
            // internal use.
            $not_available = TRUE;
          }
          // For registration, 'state' col contain string values. do not update it.
          $real_col = (!empty($info['entity admin']['properties']['status'])) ? $info['entity admin']['properties']['status'] : FALSE;
          if ($info['base table field types'][$real_col] != 'int' && is_numeric($new_value)) {
            $not_available = TRUE;
          }
        }
        else {
          if (!in_array($col, $info['schema_fields_sql']['base table'])) {
            $not_available = TRUE;
          }
        }

        // Specific cases for nodes
        if ($entity_type == 'node') {
          if (in_array($col, array('status', 'sticky', 'promote'))) {
            // A user can be allowed to edit node but not publishing it.
            if (!user_access('administer nodes')) {
              form_set_error($entity_ref, t('The operation %operation is not allowed for the %title @entity_type.', $args_update));
            }
          }
        }

      }

      // Look for entity status
      if (!empty($info['entity admin']['properties']['entity_status'])) {
        if (entity_admin_entity_has_status($entity_type, $entity, ENTITY_FIXED) || (entity_admin_entity_has_status($entity_type, $entity, ENTITY_IN_CODE) && !entity_admin_entity_has_status($entity_type, $entity, ENTITY_OVERRIDDEN))) {
          $not_available = TRUE;
        }
      }
      if ($not_available) {
        form_set_error($entity_ref, t('The operation %operation is not available for the %title @entity_type.', $args_update));
      }
    }
  }
}

/**
 * Process entity_admin_admin_entities form submissions.
 *
 * Executes the chosen 'Update option' on the selected entities. Inspired by the
 * node_admin_nodes_submit() core function.
 *
 * @see entity_admin_admin_entities()
 * @see entity_admin_admin_entities_validate()
 * @see entity_admin_filter_form()
 * @see entity_admin_filter_form_submit()
 * @see entity_admin_multiple_delete_confirm()
 * @see entity_admin_multiple_delete_confirm_submit()
 */
function entity_admin_admin_entities_submit($form, &$form_state) {

  $operations = module_invoke_all('entity_admin_operations');
  $operation = $operations[$form_state['values']['operation']];
  // Filter out unchecked entities.
  $entities = array_filter($form_state['values']['entities']);
  if ($function = $operation['callback']) {
    // Add in callback arguments if present.
    if (isset($operation['callback arguments'])) {
      $args = array_merge(array($entities), $operation['callback arguments']);
    }
    else {
      $args = array($entities);
    }
    call_user_func_array($function, $args);

    cache_clear_all();
  }
  else {
    // We need to rebuild the form to go to a second step. For example, to
    // show the confirmation form for the deletion of entities.
    $form_state['rebuild'] = TRUE;
  }
}

/**
 * Multiple entity deletion confirmation form for entity_admin_admin_content().
 *
 * Inspired by the node_multiple_delete_confirm() core function.
 *
 * @see entity_admin_admin_entities()
 * @see entity_admin_admin_entities_submit()
 * @see entity_admin_admin_entities_validate()
 * @see entity_admin_filter_form()
 * @see entity_admin_filter_form_submit()
 * @see entity_admin_multiple_delete_confirm_submit()
 * @ingroup forms
 */
function entity_admin_multiple_delete_confirm($form, &$form_state, $entities) {

  $form['entities'] = array(
    '#prefix' => '<ul>',
    '#suffix' => '</ul>',
    '#tree' => TRUE,
  );
  // array_filter returns only elements with TRUE values
  foreach ($entities as $entity_ref => $datas) {
    list($entity_type, $entity_id) = explode('-', $entity_ref, 2);
    $info = entity_admin_entity_get_info($entity_type);
    $id_field = ($entity_type != 'node_type') ? $info['entity keys']['id'] : 'type';
    $entity = $title = FALSE;
    if (!empty($info['entity keys']['label'])) {
      // node_type will load its title here.
      $title = db_query('SELECT ' . $info['entity keys']['label'] . ' FROM {' . $info['base table'] . '} WHERE ' . $id_field . ' = :id', array(':id' => $entity_id))->fetchField();

    }
    else {
      // Take more cpu. Load the entity
      $entity = entity_load_single($entity_type, $entity_id);
      $title = entity_label($entity_type, $entity);
    }

    if (!$title) {
      $title = t('@entity_type with @id ID', array('@entity_type' => $info['label'], '@id' => $entity_id));
    }

    // If the entity is a bundle, warm the user that deleting this entity will
    $is_bundle = (!empty($info['bundle of']));
    // cause deletion of more entities.
    if ($is_bundle) {
      // First retreive the bundle.
      $bundle_col = (!empty($info['entity keys']['name'])) ? $info['entity keys']['name'] : FALSE;
      if ($bundle_col) {
        if ($entity && !empty($entity->$bundle_col)) {
          $bundle = $entity->$bundle_col;
        }
        else {
          // Retreive from db.
          $bundle = db_query('SELECT ' . $bundle_col . ' FROM {' . $info['base table'] . '} WHERE ' . $id_field . ' = :id', array(':id' => $entity_id))->fetchField();
        }
        $sub_entity_type = $info['bundle of'];
        $sub_info = entity_admin_entity_get_info($sub_entity_type);
        $bundle_label = $title;
        if (isset($sub_info['bundles'][$bundle]['label'])) {
          $bundle_label = $sub_info['bundles'][$bundle]['label'];
        }
        $query = db_select($sub_info['base table']);
        if (!empty($sub_info['entity admin']['external bundle'])) {
          // The bundle is not stored in the entity table. Add extra table.
          // Concern deleting taxonomy_vocabulary
          $bundle_field = entity_admin_join_extra_table($query, $sub_entity_type);
          $query->condition($bundle_field, $bundle);
        }
        else {
          $query->condition($sub_info['entity keys']['bundle'], $bundle);
        }
        $count_sub_entities = $query->countQuery()->execute()->fetchField();
        $args = array(
          '@entity_type' => $info['label'],
          '%type' => $bundle_label,
          '%subentities' => (!empty($sub_info['plural label'])) ? $sub_info['plural label'] : $sub_info['label'],
          '%subentity' => $sub_info['label'],
        );
        if ($count_sub_entities > 0) {
          // Text from node_type_delete_confirm();
          $warning = format_plural($count_sub_entities, '@entity_type %type is used by 1 piece of %subentity on your site. If you remove this @entity_type, you will not be able to edit this %subentity and it may not display correctly.', '@entity_type %type is used by @count pieces of %subentity on your site. If you remove %type, you will not be able to edit the %type %subentities and they may not display correctly.', $args);
          drupal_set_message('<strong>' . $warning . '</strong>', 'warning');
        }
      }
    }

    $entity_display_title = t('@entity_type : %title', array('@entity_type' => $info['label'], '%title' => $title));
    if ($is_bundle) {
      $entity_display_title = '<strong>' . $entity_display_title . '</strong>';
    }

    $form['entities'][$entity_ref] = array(
      '#type' => 'hidden',
      '#value' => $entity_ref,
      '#prefix' => '<li>',
      '#suffix' => $entity_display_title . "</li>\n",
    );
  }
  $form['operation'] = array(
    '#type' => 'hidden',
    '#value' => 'delete',
  );
  $form['#submit'][] = 'entity_admin_multiple_delete_confirm_submit';
  $confirm_question = format_plural(count($entities),
    'Are you sure you want to delete this item?',
    'Are you sure you want to delete these items?');
  return confirm_form($form,
    $confirm_question,
    'admin/content/entities', t('This action cannot be undone.'),
    t('Delete'), t('Cancel'));
}

/**
 * Form submission handler for entity_admin_multiple_delete_confirm().
 *
 * Inspired by the node_multiple_delete_confirm_submit() core function.
 *
 * @see entity_admin_admin_entities()
 * @see entity_admin_admin_entities_submit()
 * @see entity_admin_admin_entities_validate()
 * @see entity_admin_filter_form()
 * @see entity_admin_filter_form_submit()
 * @see entity_admin_multiple_delete_confirm()
 */
function entity_admin_multiple_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $to_delete = array();
    foreach ($form_state['values']['entities'] as $entity_ref => $datas) {
      // Group entities by entity types
      list($entity_type, $entity_id) = explode('-', $entity_ref, 2);
      $to_delete[$entity_type][] = $entity_id;
    }

    foreach ($to_delete as $entity_type => $ids) {
      $info = entity_admin_entity_get_info($entity_type);
      if (!empty($info['entity admin']['delete multiple callback'])) {
        // Use an entity_type specific callback.
        $info['entity admin']['delete multiple callback']($ids);
      }
      else {
        if (!empty($info['entity admin']['not entity'])) {
          $function = (!empty($info['deletion callback'])) ? $info['deletion callback'] : FALSE;
          if ($function) {
            // Delete one by one.
            foreach ($ids as $id) {
              call_user_func($function, $id);
            }
          }
        }
        else {
          entity_delete_multiple($entity_type, $ids);
        }
      }
    }
    $count = count($form_state['values']['entities']);
    watchdog('entity_admin', 'Deleted @count entities.', array('@count' => $count));
    drupal_set_message(format_plural($count, 'Deleted 1 entity.', 'Deleted @count entities.'));
  }
  $form_state['redirect'] = 'admin/content/entities';
}
